local name, addon = ...
local _G = _G
local pairs = pairs
local sort = sort
local buttons, securebuttons = {}, {}
addon['sortbuttons'] = addon['sortbuttons'] or {}
addon['tooltips'] = addon['tooltips'] or {}

local MSQ = LibStub("Masque",true)
if(MSQ) then
	local MSQGroup = MSQ:Group(name, name)
end

local function buttonname(bar, aurainfo)
	local n = 1
	while(_G[format("%sButton%s",name,n)]) do
		n = n + 1
	end
	return format("%sButton%s",name,n)
end

local function isSecure(bar,aurainfo)
	return bar['spells'][aurainfo[11]] and bar['spells'][aurainfo[11]]['sticky'] or nil
end

local nextUp,nextDelay = 0,.5
local function tooltipUpdate() end  -- dummy

local function tooltip(self)
	GameTooltip:SetOwner(self,"ANCHOR_CURSOR")
	GameTooltip:ClearLines()
	if(self['aura'][11]) then
		if(type(self['aura'][11]) == 'string' and type(addon['tooltips'][self['aura'][11]]) == 'function') then
			addon['tooltips'][self['aura'][11]](self)
		elseif(type(self['aura'][11]) == 'number') then
			if(not self['aura'][#self['aura']-1]) then
				GameTooltip:SetSpellByID(self['aura'][11])
			else
				GameTooltip:SetUnitAura(self['aura'][#self['aura']], self['aura'][#self['aura']-1], self['aura'][#self['aura']-2])
			end
		else
			GameTooltip:AddLine(self['aura'][1])
		end
	end
	GameTooltip:Show()
	self:SetScript("OnUpdate", tooltipUpdate)
end

function tooltipUpdate(self) -- local, see dummy above
	if(nextUp>GetTime()) then return end
	nextUp = GetTime() + nextDelay
	if(GameTooltip:IsOwned(self)) then tooltip(self) end
end

local function tooltip_hide(self)
	if(GameTooltip:IsOwned(self)) then GameTooltip:Hide() self:SetScript("OnUpdate", nil) end
end

local function createButton(bar,bname,secure)
	if(not secure) then
		for i = #buttons,1,-1 do
			if(not buttons[i]:IsVisible()) then
				return buttons[i]
			end
		end
	end
	local button = CreateFrame("Button",bname, UIParent,secure and "SecureActionButtonTemplate ActionButtonTemplate" or "ActionButtonTemplate")
	if(secure) then
		securebuttons[#securebuttons + 1] = button
	else
		buttons[#buttons + 1] = button
	end
	if(MSQ and MSQGroup) then
		MSQGroup:AddButton(button)
	end
	return button
end

local function sortBar(a, b)
	if(a and b and a['aura'][7]==0 and b['aura'][7]==0) then
		if(type(a['aura'][11]) == 'string' or type(b['aura'][11] == 'string')) then
			if(type(addon['sortbuttons'][type(a['aura'][11]) == 'string' and a['aura'][11] or b['aura'][11]])=='function') then
				return addon['sortbuttons'][type(a['aura'][11]) == 'string' and a['aura'][11] or b['aura'][11]](a, b)
			end
		end		
		return a['aura'][11] > b['aura'][11]
	end
	return a['aura'][7] > b['aura'][7]
end

addon['updateButton'] = function(bar, aurainfo)
-- name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId, canApplyAura, isBossDebuff, value1, value2, value3
	local x, y = bar['vertical'] and 0 or bar['stickybuttons'], bar['vertical'] and bar['stickybuttons'] or 0
	sort(bar['buttons'], sortBar)
	for i = 1,#bar['buttons'] do
--	print(i,bar['buttons'][i]['aura'][1])
		if(bar['buttons'][i]:IsVisible()) then
			if(not isSecure(bar,bar['buttons'][i]['aura'])) then
				local w, h = bar['buttons'][i]:GetWidth(),bar['buttons'][i]:GetHeight()
				local d = bar['reverse'] and -1 or 1
				bar['buttons'][i]:SetPoint("CENTER", x*w*(bar['vertical'] and 1 or d),y*h*(bar['vertical'] and d or 1))
				if(bar['vertical']) then
					y = y + 1
				else
					x = x + 1
				end
			end
			if(bar['buttons'][i]['aura'][11] == aurainfo[11]) then
				bar['buttons'][i]['aura'] = aurainfo
				if(type(bar['buttons'][i]['aura'][#bar['buttons'][i]['aura'] - 1]) == 'number' or (type(addon['AuraIsUp'][bar['buttons'][i]['aura'][11]])=='function' and addon['AuraIsUp'][bar['buttons'][i]['aura'][11]]())) then
					bar['buttons'][i]['Icon']:SetDesaturated(false)
				else
					bar['buttons'][i]['Icon']:SetDesaturated(true)
				end
				
				bar['buttons'][i]['Icon']:SetTexture(aurainfo[3] or "Interface\\Icons\\Ability_Warrior_PunishingBlow")
				bar['buttons'][i]['Count']:SetText(aurainfo[4] > 1 and aurainfo[4] or '')
				
				if(aurainfo[6] > 0) then
					bar['buttons'][i]['CooldownDir'] = true
					bar['buttons'][i]['Cooldown']:SetReverse(true)
					CooldownFrame_SetTimer(bar['buttons'][i]['Cooldown'], aurainfo[7] - aurainfo[6], aurainfo[6], 1)
				elseif(bar['buttons'][i]['CooldownDir']) then
					bar['buttons'][i]['Cooldown']:SetReverse(false)
					bar['buttons'][i]['CooldownDir'] = false
					CooldownFrame_SetTimer(bar['buttons'][i]['Cooldown'], 360, 360, 1)
				end
			end
		else
			bar['buttons'][i] = nil
		end
	end
end

addon['remButton'] = function(bar, aurainfo, force)
	if(not aurainfo[1]) then print("not found",bar['name']) return end
--	print("rem",  buttonname(bar, aurainfo))
	for i = #bar['buttons'],1,-1 do
		if(bar['buttons'][i]['aura'][11] == aurainfo[11]) then
			if(force or not isSecure(bar,aurainfo)) then
				bar['buttons'][i]:Hide()
				bar['auras'][aurainfo[11]] = nil
				table.remove(bar['buttons'],i)
			else
				bar['auras'][aurainfo[11]] = nil
				bar['buttons'][i]['aura'][#bar['buttons'][i]['aura'] - 1] = false
				bar['buttons'][i]['Icon']:SetDesaturated(true)
			end
		end
	end
	addon['updateButton'](bar, aurainfo)
end
local function OnClick(self, button)
	if(InCombatLockdown()) then return end
end
addon['addButton'] = function(bar, aurainfo)
--	print("add", aurainfo[1], buttonname(bar, aurainfo))
	local button, reuse = createButton(bar, buttonname(bar, aurainfo),isSecure(bar,aurainfo))
	if(not reuse) then
		bar['buttons'][#bar['buttons'] + 1] = button
	end
	button['aura'] = aurainfo
	button['Icon'] = button['Icon'] or _G[format("%sIcon",button:GetName())]
	button['Count'] = button['Count'] or _G[format("%sCount",button:GetName())]
	button['Cooldown'] = button['Cooldown'] or _G[format("%sCooldown",button:GetName())]
	button['Icon']:SetTexture(aurainfo[3] or "Interface\\Icons\\Ability_Warrior_PunishingBlow")
	button:SetScript("OnEnter", tooltip)
	button:SetScript("OnLeave", tooltip_hide)
	bar['auras'][aurainfo[11]] = aurainfo
	if(not isSecure(bar,aurainfo)) then
		button:SetParent(bar['frame'])
		button:Show()
		button:SetScript("OnClick", OnClick)
		button:RegisterForClicks("AnyUp")
	end
	addon['updateButton'](bar, aurainfo)
end

